
class RotorSE_Example3():

	def execute(self):

		# --- Import Modules
		import numpy as np
		import os
		from openmdao.api import IndepVarComp, Component, Group, Problem, Brent, ScipyGMRES, ScipyOptimizer, DumpRecorder
		from rotorse.rotor_aeropower import RotorAeroPower
		from rotorse.rotor_geometry import RotorGeometry, NREL5MW, DTU10MW, NINPUT
		from rotorse import RPM2RS, RS2RPM, TURBULENCE_CLASS, DRIVETRAIN_TYPE
		from rotorse.rotor import RotorSE

		myref = DTU10MW()

		rotor = Problem()
		npts_coarse_power_curve = 20 # (Int): number of points to evaluate aero analysis at
		npts_spline_power_curve = 200  # (Int): number of points to use in fitting spline to power curve

		rotor.root = RotorSE(myref, npts_coarse_power_curve, npts_spline_power_curve)
		rotor.setup()

		# ---
		# === blade grid ===
		rotor['hubFraction'] = myref.hubFraction #0.023785  # (Float): hub location as fraction of radius
		rotor['bladeLength'] = myref.bladeLength #96.7  # (Float, m): blade length (if not precurved or swept) otherwise length of blade before curvature
		rotor['precone'] = myref.precone #4.  # (Float, deg): precone angle
		rotor['tilt'] = myref.tilt #6.0  # (Float, deg): shaft tilt
		rotor['yaw'] = 0.0  # (Float, deg): yaw error
		rotor['nBlades'] = myref.nBlades #3  # (Int): number of blades
		# ---

		# === blade geometry ===
		rotor['r_max_chord'] =  myref.r_max_chord  # 0.2366 #(Float): location of max chord on unit radius
		rotor['chord_in'] = myref.chord # np.array([4.6, 4.869795, 5.990629, 3.00785428, 0.0962])  # (Array, m): chord at control points. defined at hub, then at linearly spaced locations from r_max_chord to tip
		rotor['theta_in'] = myref.theta # np.array([14.5, 12.874, 6.724, -0.03388039, -0.037]) # (Array, deg): twist at control points.  defined at linearly spaced locations from r[idx_cylinder] to tip
		rotor['precurve_in'] = myref.precurve #np.array([-0., -0.054497, -0.175303, -0.84976143, -6.206217])  # (Array, m): precurve at control points.  defined at same locations at chord, starting at 2nd control point (root must be zero precurve)
		rotor['presweep_in'] = myref.presweep #np.array([0., 0., 0., 0., 0.])  # (Array, m): precurve at control points.  defined at same locations at chord, starting at 2nd control point (root must be zero precurve)
		rotor['sparT_in'] = myref.spar_thickness # np.array([0.03200042 0.07038508 0.08515644 0.07777004 0.01181032])  # (Array, m): spar cap thickness parameters
		rotor['teT_in'] = myref.te_thickness # np.array([0.04200055 0.08807739 0.05437378 0.01610219 0.00345225])  # (Array, m): trailing-edge thickness parameters
		# ---

		# === atmosphere ===
		rotor['analysis.rho'] = 1.225  # (Float, kg/m**3): density of air
		rotor['analysis.mu'] = 1.81206e-5  # (Float, kg/m/s): dynamic viscosity of air
		rotor['wind.shearExp'] = 0.25  # (Float): shear exponent
		rotor['hub_height'] = myref.hub_height #119.0  # (Float, m): hub height
		rotor['turbine_class'] = myref.turbine_class #TURBINE_CLASS['I']  # (Enum): IEC turbine class
		rotor['turbulence_class'] = TURBULENCE_CLASS['B']  # (Enum): IEC turbulence class class
		rotor['wind.zref'] = myref.hub_height #119.0  # (Float): reference hub height for IEC wind speed (used in CDF calculation)
		rotor['gust_stddev'] = 3
		# ---

		# === control ===
		rotor['control_Vin'] = myref.control_Vin #4.0  # (Float, m/s): cut-in wind speed
		rotor['control_Vout'] = myref.control_Vout #25.0  # (Float, m/s): cut-out wind speed
		rotor['control_minOmega'] = myref.control_minOmega #6.0  # (Float, rpm): minimum allowed rotor rotation speed
		rotor['control_maxOmega'] = myref.control_maxOmega #8.88766  # (Float, rpm): maximum allowed rotor rotation speed
		rotor['control_tsr'] = myref.control_tsr #10.58  # (Float): tip-speed ratio in Region 2 (should be optimized externally)
		rotor['control_pitch'] = myref.control_pitch #0.0  # (Float, deg): pitch angle in region 2 (and region 3 for fixed pitch machines)
		rotor['machine_rating'] = myref.rating #10e6  # (Float, W): rated power
		rotor['pitch_extreme'] = 0.0  # (Float, deg): worst-case pitch at survival wind condition
		rotor['azimuth_extreme'] = 0.0  # (Float, deg): worst-case azimuth at survival wind condition
		rotor['VfactorPC'] = 0.7  # (Float): fraction of rated speed at which the deflection is assumed to representative throughout the power curve calculation
		# ---

		# === aero and structural analysis options ===
		rotor['nSector'] = 4  # (Int): number of sectors to divide rotor face into in computing thrust and power
		rotor['AEP_loss_factor'] = 1.0  # (Float): availability and other losses (soiling, array, etc.)
		rotor['drivetrainType'] = myref.drivetrain #DRIVETRAIN_TYPE['GEARED']  # (Enum)
		rotor['dynamic_amplication_tip_deflection'] = 1.35  # (Float): a dynamic amplification factor to adjust the static deflection calculation
		# ---


		# === fatigue ===
		r_aero = np.array([0.02222276, 0.06666667, 0.11111057, 0.2, 0.23333333, 0.3, 0.36666667, 0.43333333,
		               0.5, 0.56666667, 0.63333333, 0.64, 0.7, 0.83333333, 0.88888943, 0.93333333,
		               0.97777724])  # (Array): new aerodynamic grid on unit radius
		rstar_damage = np.array([0.000, 0.022, 0.067, 0.111, 0.167, 0.233, 0.300, 0.367, 0.433, 0.500,
		    0.567, 0.633, 0.700, 0.767, 0.833, 0.889, 0.933, 0.978])  # (Array): nondimensional radial locations of damage equivalent moments
		Mxb_damage = 1e3*np.array([2.3743E+003, 2.0834E+003, 1.8108E+003, 1.5705E+003, 1.3104E+003,
		    1.0488E+003, 8.2367E+002, 6.3407E+002, 4.7727E+002, 3.4804E+002, 2.4458E+002, 1.6339E+002,
		    1.0252E+002, 5.7842E+001, 2.7349E+001, 1.1262E+001, 3.8549E+000, 4.4738E-001])  # (Array, N*m): damage equivalent moments about blade c.s. x-direction
		Myb_damage = 1e3*np.array([2.7732E+003, 2.8155E+003, 2.6004E+003, 2.3933E+003, 2.1371E+003,
		    1.8459E+003, 1.5582E+003, 1.2896E+003, 1.0427E+003, 8.2015E+002, 6.2449E+002, 4.5229E+002,
		    3.0658E+002, 1.8746E+002, 9.6475E+001, 4.2677E+001, 1.5409E+001, 1.8426E+000])  # (Array, N*m): damage equivalent moments about blade c.s. y-direction
		xp = np.r_[0.0, r_aero]
		xx = np.r_[0.0, myref.r]
		rotor['rstar_damage'] = np.interp(xx, xp, rstar_damage)
		rotor['Mxb_damage'] = np.interp(xx, xp, Mxb_damage)
		rotor['Myb_damage'] = np.interp(xx, xp, Myb_damage)
		rotor['strain_ult_spar'] = 1.0e-2  # (Float): ultimate strain in spar cap
		rotor['strain_ult_te'] = 2500*1e-6 * 2   # (Float): uptimate strain in trailing-edge panels, note that I am putting a factor of two for the damage part only.
		rotor['gamma_fatigue'] = 1.755 # (Float): safety factor for fatigue
		rotor['gamma_f'] = 1.35 # (Float): safety factor for loads/stresses
		rotor['gamma_m'] = 1.1 # (Float): safety factor for materials
		rotor['gamma_freq'] = 1.1 # (Float): safety factor for resonant frequencies
		rotor['m_damage'] = 10.0  # (Float): slope of S-N curve for fatigue analysis
		rotor['struc.lifetime'] = 20.0  # (Float): number of cycles used in fatigue analysis  TODO: make function of rotation speed
		# ----------------

		# === run and outputs ===
		rotor.run()

		print 'AEP =', rotor['AEP']
		print 'diameter =', rotor['diameter']
		print 'rated_V =', rotor['rated_V']
		print 'rated_Omega =', rotor['rated_Omega']
		print 'rated_pitch =', rotor['rated_pitch']
		print 'rated_T =', rotor['rated_T']
		print 'rated_Q =', rotor['rated_Q']
		print 'mass_one_blade =', rotor['mass_one_blade']
		print 'mass_all_blades =', rotor['mass_all_blades']
		print 'I_all_blades =', rotor['I_all_blades']
		print 'freq =', rotor['freq']
		print 'tip_deflection =', rotor['tip_deflection']
		print 'root_bending_moment =', rotor['root_bending_moment']

		outpath = '..\..\..\docs\images'
		import matplotlib.pyplot as plt
		plt.figure()
		plt.plot(rotor['V'], rotor['P']/1e6)
		plt.xlabel('wind speed (m/s)')
		plt.xlabel('power (W)')

		plt.figure()

		plt.plot(rotor['r_pts'], rotor['strainU_spar'], label='suction')
		plt.plot(rotor['r_pts'], rotor['strainL_spar'], label='pressure')
		plt.plot(rotor['r_pts'], rotor['eps_crit_spar'], label='critical')
		# plt.ylim([-6e-3, 6e-3])
		plt.xlabel('r')
		plt.ylabel('strain')
		plt.legend()
		plt.savefig(os.path.abspath(os.path.join(outpath,'strain_spar_dtu10mw.png')))
		plt.savefig(os.path.abspath(os.path.join(outpath,'strain_spar_dtu10mw.pdf')))

		plt.figure()

		plt.plot(rotor['r_pts'], rotor['strainU_te'], label='suction')
		plt.plot(rotor['r_pts'], rotor['strainL_te'], label='pressure')
		plt.plot(rotor['r_pts'], rotor['eps_crit_te'], label='critical')
		# plt.ylim([-5e-3, 5e-3])
		plt.xlabel('r')
		plt.ylabel('strain')
		plt.legend()
		plt.savefig(os.path.abspath(os.path.join(outpath,'strain_te_dtu10mw.png')))
		plt.savefig(os.path.abspath(os.path.join(outpath,'strain_te_dtu10mw.pdf')))

		plt.show()
		# ----------------


if __name__ == "__main__":

	rotor = RotorSE_Example3()
	rotor.execute()



#!/usr/bin/env python
# encoding: utf-8
"""
test_turbine_costsse.py

Created by Katherine Dykes on 2014-01-07.
Copyright (c) NREL. All rights reserved.
"""

import unittest
from commonse.utilities import check_gradient_unit_test

from turbine_costsse.turbine_costsse import TowerCostAdder, TowerCost
from turbine_costsse.turbine_costsse import BladeCost, HubCost, PitchSystemCost, SpinnerCost, \
    HubSystemCostAdder, RotorCostAdder
from turbine_costsse.turbine_costsse import LowSpeedShaftCost, BearingsCost, GearboxCost, \
    HighSpeedSideCost, GeneratorCost, BedplateCost, \
    YawSystemCost, NacelleSystemCostAdder
from turbine_costsse.turbine_costsse import TurbineCostAdder

from turbine_costsse.nrel_csm_tcc import tower_csm_component
from turbine_costsse.nrel_csm_tcc import blades_csm_component
from turbine_costsse.nrel_csm_tcc import hub_csm_component
from turbine_costsse.nrel_csm_tcc import nacelle_csm_component
from turbine_costsse.nrel_csm_tcc import rotor_mass_adder, tcc_csm_component

# turbine_costsse Model
# ----------------------------------------------------------
# Tower Components


class TestTowerCost(unittest.TestCase):

    def test1(self):

        tower = TowerCost()

        tower.tower_mass = 434559.0
        tower.year = 2009
        tower.month = 12

        check_gradient_unit_test(self, tower)


class TestTowerCostAdder(unittest.TestCase):

    def test1(self):

        tower = TowerCostAdder()

        tower.tower_cost = 1000000.0

        check_gradient_unit_test(self, tower)

# Rotor Components


class TestBladeCost(unittest.TestCase):

    def test1(self):

        blade = BladeCost()

        blade.blade_mass = 17650.67
        blade.year = 2009
        blade.month = 12

        check_gradient_unit_test(self, blade)


class TestHubCost(unittest.TestCase):

    def test1(self):

        hub = HubCost()

        hub.hub_mass = 31644.5
        hub.year = 2009
        hub.month = 12

        check_gradient_unit_test(self, hub)


class TestPitchSystemCost(unittest.TestCase):

    def test1(self):

        pitch = PitchSystemCost()

        pitch.pitch_system_mass = 17004.0
        pitch.year = 2009
        pitch.month = 12

        check_gradient_unit_test(self, pitch)


class TestSpinnerCost(unittest.TestCase):

    def test1(self):

        spinner = SpinnerCost()

        spinner.spinner_mass = 1810.5
        spinner.year = 2009
        spinner.month = 12

        check_gradient_unit_test(self, spinner)


class TestHubSystemCostAdder(unittest.TestCase):

    def test1(self):

        hub = HubSystemCostAdder()

        hub.hub_cost = 20000.0
        hub.pitch_system_cost = 20000.0
        hub.spinner_cost = 20000.0

        check_gradient_unit_test(self, hub)


class TestRotorCostAdder(unittest.TestCase):

    def test1(self):

        rotor = RotorCostAdder()

        rotor.blade_cost = 20000.0
        rotor.blade_number = 3
        rotor.hub_system_cost = 20000.0

        check_gradient_unit_test(self, rotor)


# Nacelle Components
class TestLowSpeedShaftCost(unittest.TestCase):

    def test1(self):

        lss = LowSpeedShaftCost()

        lss.low_speed_shaft_mass = 31257.3
        lss.year = 2009
        lss.month = 12

        check_gradient_unit_test(self, lss)


class TestBearingsCost(unittest.TestCase):

    def test1(self):

        bearings = BearingsCost()

        bearings.main_bearing_mass = 9731.41 / 2.0
        bearings.second_bearing_mass = 9731.41 / 2.0
        bearings.year = 2009
        bearings.month = 12

        check_gradient_unit_test(self, bearings)


class TestGearboxCost(unittest.TestCase):

    def test1(self):

        gearbox = GearboxCost()

        gearbox.gearbox_mass = 30237.60
        gearbox.year = 2009
        gearbox.month = 12
        gearbox.drivetrain_design = 'geared'

        check_gradient_unit_test(self, gearbox)


class TestHighSpeedSideCost(unittest.TestCase):

    def test1(self):

        hss = HighSpeedSideCost()

        hss.high_speed_side_mass = 1492.45
        hss.year = 2009
        hss.month = 12

        check_gradient_unit_test(self, hss)


class TestGeneratorCost(unittest.TestCase):

    def test1(self):

        generator = GeneratorCost()

        generator.generator_mass = 16699.85
        generator.year = 2009
        generator.month = 12
        generator.drivetrain_design = 'geared'
        generator.machine_rating = 5000.0

        check_gradient_unit_test(self, generator)


class TestBedplateCost(unittest.TestCase):

    def test1(self):

        bedplate = BedplateCost()

        bedplate.bedplate_mass = 93090.6
        bedplate.year = 2009
        bedplate.month = 12

        check_gradient_unit_test(self, bedplate)


class TestYawSystemCost(unittest.TestCase):

    def test1(self):

        yaw = YawSystemCost()

        yaw.yaw_system_mass = 11878.24
        yaw.year = 2009
        yaw.month = 12

        check_gradient_unit_test(self, yaw)


class TestNacelleSystemCostAdder(unittest.TestCase):

    def test1(self):

        nacelle = NacelleSystemCostAdder()

        nacelle.bedplate_mass = 93090.6
        nacelle.machine_rating = 5000.0
        nacelle.drivetrainDesign = 1
        nacelle.crane = True
        nacelle.offshore = True
        nacelle.year = 2009
        nacelle.month = 12
        nacelle.lss_cost = 10000.0
        nacelle.bearings_cost = 10000.0
        nacelle.gearbox_cost = 10000.0
        nacelle.hss_cost = 10000.0
        nacelle.bedplate_cost = 10000.0
        nacelle.bedplateCost2002 = 8000.0
        nacelle.yaw_system_cost = 10000.0

        check_gradient_unit_test(self, nacelle, tol=1e-5)

# Turbine Components


class TestTurbineCostAdder(unittest.TestCase):

    def test1(self):

        turbine = TurbineCostAdder()

        turbine.offshore = True
        turbine.rotor_cost = 2000000.0
        turbine.nacelle_cost = 5000000.0
        turbine.tower_cost = 1000000.0

        check_gradient_unit_test(self, turbine)


# NREL CSM TCC Components
# ----------------------------------------------------------
# Tower Component
class Test_tower_csm_component(unittest.TestCase):

    def test1(self):

        tower = tower_csm_component()

        tower.rotor_diameter = 126.0
        tower.hub_height = 90.0
        tower.year = 2009
        tower.month = 12
        tower.advanced = False

        check_gradient_unit_test(self, tower)

# Rotor Components


class Test_blade_csm_component(unittest.TestCase):

    def test1(self):

        blades = blades_csm_component()

        blades.rotor_diameter = 126.0
        blades.advanced_blade = False
        blades.year = 2009
        blades.month = 12
        blades.advanced_blade = False

        check_gradient_unit_test(self, blades)


class Test_hub_csm_component(unittest.TestCase):

    def test1(self):

        hub = hub_csm_component()

        hub.blade_mass = 25614.377
        hub.rotor_diameter = 126.0
        hub.blade_number = 3
        hub.year = 2009
        hub.month = 12

        check_gradient_unit_test(self, hub)

# Nacelle Components


class Test_nacelle_csm_component(unittest.TestCase):

    def test1(self):

        nac = nacelle_csm_component()

        nac.rotor_diameter = 126.0
        nac.machine_rating = 5000.0
        nac.rotor_mass = 123193.30
        nac.rotor_thrust = 500930.1
        nac.rotor_torque = 4365249
        nac.drivetrain_design = 'geared'
        nac.offshore = True
        nac.crane = True
        nac.advanced_bedplate = 0
        nac.year = 2009
        nac.month = 12

        check_gradient_unit_test(self, nac, display=False)

# Turbine Components


class Test_rotor_mass_adder(unittest.TestCase):

    def test1(self):

        rotor = rotor_mass_adder()

        rotor.blade_mass = 17000.
        rotor.hub_system_mass = 35000.
        rotor.blade_number = 3

        check_gradient_unit_test(self, rotor)


# Turbine Components
class Test_tcc_csm_component(unittest.TestCase):

    def test1(self):

        trb = tcc_csm_component()

        trb.rotor_diameter = 126.0
        trb.advanced_blade = True
        trb.blade_number = 3
        trb.hub_height = 90.0
        trb.machine_rating = 5000.0
        trb.offshore = True
        trb.year = 2009
        trb.month = 12

        check_gradient_unit_test(self, trb)

#----------------------------------------------------

if __name__ == "__main__":
    unittest.main()



"""
config.py

Created by George Scott on 2012-08-01.
Modified by Katherine Dykes 2012.
Copyright (c) NREL. All rights reserved.
"""

""" config file for global variables """

from csmPPI import *

# Initialize ref and current YYYYMM
# Calling program can override these
#   e.g., ppi.ref_yr = 2003, etc.

ref_yr  = 2002
ref_mon =    9
curr_yr = 2009
curr_mon =  12

ppi = PPI(ref_yr,ref_mon,curr_yr,curr_mon)



#-------------------------------------------------------------------------
# Name:        SegIntersect.py
# Purpose: It Calculates Intersection of 2 segments in space. Also contains a simple 2-point distance calculator.
#
# Author:      RRD
#
# Created:     24/10/2012
# Copyright:   (c) rdamiani 2012
# Licence:     <your licence>
#-------------------------------------------------------------------------


import numpy as np

def CalcDist(X1,X2):
    """This function returns the distance (norm) between the two points X1(x1,y1,z1) and X2(x2,y2,z2).
    X1 and X2 can be [3,n] arrays."""
    junk=X2-X1
    return np.sqrt(np.sum(junk**2,0))


def SegIntersect(A1, A2, B1, B2):
    """The function returns the intersection or the points of closest approach if lines are skewed.
    If lines are parallel, NaN is returned.
    INPUT:
        A1  -float(3,n), [x,y,z;nsegments] cordinates of 1st point(s) of 1st segment(s)
        A2  -float(3,n), [x,y,z;nsegments] cordinates of 2nd point(s) of 1st segment(s)
        B1  -float(3,n), [x,y,z;nsegments] cordinates of 1st point(s) of 2nd segment(s)
        B2  -float(3,n), [x,y,z;nsegments] cordinates of 2nd point(s) of 2nd segment(s)
    OUTPUT:
        A0  -float(3,n), [x,y,z;nsegments] coordinates of intersection point (=B0) or closet point to 2nd line on 1st segment,
        B0  -float(3,n), [x,y,z;nsegments] coordinates of intersection point (=A0) or closet point to 1st line on 2nd segment,
        OR  -NaN
    """

    #reshape A1..B2 in case they have 1 dimension only
    A1=A1.reshape(3,-1)
    A2=A2.reshape(3,-1)
    B1=B1.reshape(3,-1)
    B2=B2.reshape(3,-1)

    vec = np.cross(A2 - A1, B2 - B1, 0, 0, 0)
    nA = np.sum(np.cross(B2 - B1, A1 - B1, 0, 0, 0) * vec, axis=0)*np.ones(A1.shape[1])
    nB = np.sum(np.cross(A2 - A1, A1 - B1, 0, 0, 0) * vec, axis=0)*np.ones(A1.shape[1])
    d = np.sum(vec**2, axis=0)*np.ones(A1.shape[1])

    A0 = np.ones(A1.shape) * np.NaN
    B0 = A0.copy()
    idx = np.nonzero(d)[0]
    A0[:, idx] = A1[:, idx] + (nA[idx] / d[idx]) * (A2[:, idx] - A1[:, idx])
    B0[:, idx] = B1[:, idx] + (nB[idx] / d[idx]) * (B2[:, idx] - B1[:, idx])

    return A0, B0

if __name__ == '__main__':
    A1=np.array([-5.939,	-5.939,	-43.127])
    A2=np.array([4.016,	-4.016,	15.651])
    B1=np.array([-4.016,	-4.016,	15.651])
    B2=np.array([5.939,	-5.939,	-43.127])

#or
    A1=np.array([-5.939,	-5.939,	-50.001])
    A2=np.array([4.016,	-4.016,	15.651])
    B2=np.array([-4.016,	-4.016,	15.651])
    B1=np.array([5.939,	-5.939,	-50.001])


    print ('Intersection coordinates='+3*'{:8.4f}, ').format(*SegIntersect(A1, A2, B1, B2)[0].flatten())

    #main()



#-------------------------------------------------------------------------------
# Name:        axisEqual3D.py
# Purpose:    When called with an axis3D instance it will render aspect ratio =1:1:1
#
# Author:      rdamiani- from http://stackoverflow.com/questions/8130823/set-matplotlib-3d-plot-aspect-ratio
#
# Created:     08/12/2013
# Copyright:   (c) rdamiani 2013
# Licence:     <your licence>
#-------------------------------------------------------------------------------
import numpy as np

def main():
    pass

if __name__ == '__main__':
    main()

def axisEqual3D(ax):
    extents = np.array([getattr(ax, 'get_{}lim'.format(dim))() for dim in 'xyz'])
    sz = extents[:,1] - extents[:,0]
    centers = np.mean(extents, axis=1)
    maxsize = max(abs(sz))
    r = maxsize/2
    for ctr, dim in zip(centers, 'xyz'):
        getattr(ax, 'set_{}lim'.format(dim))(ctr - r, ctr + r)



import numpy as np
import numpy.testing as npt
import unittest
from wisdem.commonse.tube import Tube, CylindricalShellProperties

npts = 100

class TestTube(unittest.TestCase):
    def setUp(self):
        self.params = {}
        self.unknowns = {}
        self.resid = None

        self.params['d'] = 2*5.0*np.ones(npts)
        self.params['t'] = 1.0*np.ones(npts-1)
        
        self.mytube = CylindricalShellProperties(nFull=npts)


    def testTubeProperties(self):
        T = Tube(2*5.0, 1.0)
        
        self.assertAlmostEqual(T.Area, np.pi*9.0)
        self.assertAlmostEqual(T.Jxx,  np.pi*369.0/4.0)
        self.assertAlmostEqual(T.Jyy,  np.pi*369.0/4.0)
        self.assertAlmostEqual(T.J0,  np.pi*369.0/2.0)
        self.assertAlmostEqual(T.S,  np.pi*369.0/4.0/5.0)
        self.assertAlmostEqual(T.C,  np.pi*369.0/2.0/5.0)
    
    def testOutputsIncremental(self):
        self.mytube.compute(self.params, self.unknowns)

        npt.assert_almost_equal(self.unknowns['Az'], np.pi*9.0)
        npt.assert_almost_equal(self.unknowns['Ixx'],  np.pi*369.0/4.0)
        npt.assert_almost_equal(self.unknowns['Iyy'],  np.pi*369.0/4.0)
        npt.assert_almost_equal(self.unknowns['Jz'],  np.pi*369.0/2.0)
        
def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestTube))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner().run(suite())



#!/usr/bin/env python
# encoding: utf-8
"""
test_environment_gradients.py

Created by Andrew Ning on 2013-12-18.
Copyright (c) NREL. All rights reserved.
"""


import unittest
import numpy as np
from wisdem.commonse.utilities import check_gradient
from wisdem.commonse.environment import PowerWind, LogWind, LinearWaves, TowerSoil
from openmdao.api import ScipyOptimizeDriver, Problem, Group, IndepVarComp


class TestPowerWind(unittest.TestCase):


    def setUp(self):

        z = np.linspace(0.0, 100.0, 20)
        nPoints = len(z)
        Uref = 10.0
        zref = 100.0
        z0 = 0.001 #Fails when z0 = 0, What to do here?
        shearExp = 0.2
        betaWind = 0.0

        prob = Problem()
        root = prob.model = Group()
        root.add_subsystem('p1', IndepVarComp('z', z, units='m'))
        root.add_subsystem('p2', IndepVarComp('zref', zref, units='m'))
        root.add_subsystem('p3', IndepVarComp('Uref', Uref, units='m/s'))
        root.add_subsystem('p', PowerWind(nPoints=nPoints))


        root.connect('p1.z', 'p.z')
        root.connect('p2.zref', 'p.zref')
        root.connect('p3.Uref', 'p.Uref')

        prob.driver = ScipyOptimizeDriver()
        prob.model.add_objective('p.U', scaler=1E-6)

        prob.model.add_design_var('p1.z', lower=np.ones(nPoints), upper=np.ones(nPoints)*1000, scaler=1E-6)
        prob.model.add_design_var('p2.zref', lower=0, upper=1000, scaler=1E-6)
        prob.model.add_design_var('p3.Uref', lower=0, upper=1000, scaler=1E-6)

        prob.setup()

        prob['p.z0'] = z0
        prob['p.shearExp'] = shearExp
        #prob['p.betaWind'] = betaWind

        prob.run_driver()

        print(prob['p.U'])

        self.J = prob.check_total_derivatives(out_stream=None)

    def test_U(self):
        np.testing.assert_allclose(self.J[('p.U', 'p3.Uref')]['J_fwd'], self.J[('p.U', 'p3.Uref')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('p.U', 'p2.zref')]['J_fwd'], self.J[('p.U', 'p2.zref')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('p.U', 'p1.z')]['J_fwd'], self.J[('p.U', 'p1.z')]['J_fd'], 1e-6, 1e-6)



class TestLogWind(unittest.TestCase):


    def setUp(self):

        z = np.linspace(0.0, 100.0, 20)
        nPoints = len(z)
        Uref = 10.0
        zref = 100.0
        z0 = 0. #Fails when z0 = 0
        betaWind = 0.0

        prob = Problem()
        root = prob.model = Group()
        root.add_subsystem('p1', IndepVarComp('z', z, units='m'))
        root.add_subsystem('p2', IndepVarComp('zref', Uref, units='m'))
        root.add_subsystem('p3', IndepVarComp('Uref', zref, units='m/s'))
        root.add_subsystem('p', LogWind(nPoints=nPoints))


        root.connect('p1.z', 'p.z')
        root.connect('p2.zref', 'p.zref')
        root.connect('p3.Uref', 'p.Uref')

        prob.driver = ScipyOptimizeDriver()
        prob.model.add_objective('p.U', scaler=1E-6)

        prob.model.add_design_var('p1.z', lower=np.ones(nPoints), upper=np.ones(nPoints)*1000, scaler=1E-6)
        prob.model.add_design_var('p2.zref', lower=0, upper=1000, scaler=1E-6)
        prob.model.add_design_var('p3.Uref', lower=0, upper=1000, scaler=1E-6)

        prob.setup()

        prob['p.z0'] = z0
        #prob['p.betaWind'] = betaWind

        prob.run_driver()

        print(prob['p.U'])

        self.J = prob.check_total_derivatives(out_stream=None)

    def test_U(self):
        np.testing.assert_allclose(self.J[('p.U', 'p3.Uref')]['J_fwd'], self.J[('p.U', 'p3.Uref')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('p.U', 'p2.zref')]['J_fwd'], self.J[('p.U', 'p2.zref')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('p.U', 'p1.z')]['J_fwd'], self.J[('p.U', 'p1.z')]['J_fd'], 1e-6, 1e-6)


class TestLinearWave(unittest.TestCase):


    def setUp(self):

        Uc = 7.0
        z_surface = 20.0
        hs = 10.0
        T = 2.0
        z_floor = 0.1
        betaWave = 3.0
        z = np.linspace(z_floor, z_surface, 20)
        nPoints = len(z)

        prob = Problem()
        root = prob.model = Group()
        root.add_subsystem('p1', IndepVarComp('z', z, units='m'))
        root.add_subsystem('p2', IndepVarComp('Uc', Uc, units='m/s'))
        root.add_subsystem('p', LinearWaves(nPoints=nPoints))

        root.connect('p1.z', 'p.z')
        root.connect('p2.Uc', 'p.Uc')

        prob.driver = ScipyOptimizeDriver()
        prob.model.add_objective('p.U', scaler=1E-6)

        prob.model.add_design_var('p1.z', lower=np.ones(nPoints), upper=np.ones(nPoints)*1000, scaler=1E-6)
        prob.model.add_design_var('p2.Uc', lower=0, upper=1000, scaler=1E-6)

        prob.setup()

        prob['p.z_surface'] = z_surface
        prob['p.z_floor'] = z_floor
        prob['p.hmax'] = hs
        prob['p.T'] = T
        #prob['p.betaWave'] = betaWave

        prob.run_driver()

        print(prob['p.U'])

        self.J = prob.check_total_derivatives(out_stream=None)

        print(self.J)

    def test_U(self):

        np.testing.assert_allclose(self.J[('p.U', 'p2.Uc')]['J_fwd'], self.J[('p.U', 'p2.Uc')]['J_fd'], 1e-6, 1e-6)
        #np.testing.assert_allclose(self.J[('p.U', 'p1.z')]['J_fwd'], self.J[('p.U', 'p1.z')]['J_fd'], 1e-6, 1e-6)


class TestSoil(unittest.TestCase):

    def setUp(self):

        d0 = 10.0
        depth = 30.0
        G = 140e6
        nu = 0.4
        rigid = [False, False, False, False, False, False]

        prob = Problem()
        root = prob.model = Group()
        root.add_subsystem('p1', IndepVarComp('d0', d0, units='m'))
        root.add_subsystem('p2', IndepVarComp('depth', depth, units='m'))
        root.add_subsystem('p', TowerSoil())

        root.connect('p1.d0', 'p.d0')
        root.connect('p2.depth', 'p.depth')

        prob.driver = ScipyOptimizeDriver()
        prob.model.add_objective('p.k', scaler=1E-6)

        prob.model.add_design_var('p1.d0', lower=0, upper=1000, scaler=1E-6)
        prob.model.add_design_var('p2.depth', lower=0, upper=1000, scaler=1E-6)

        prob.setup()

        prob['p.G'] = G
        prob['p.nu'] = nu

        prob.run_driver()

        self.J = prob.check_total_derivatives(out_stream=None)

        print(self.J)

    def test_k(self):

        np.testing.assert_allclose(self.J[('p.k', 'p1.d0')]['J_fwd'], self.J[('p.k', 'p1.d0')]['J_fd'], 1e-6, 1e-6)
        np.testing.assert_allclose(self.J[('p.k', 'p1.depth')]['J_fwd'], self.J[('p.k', 'p1.depth')]['J_fd'], 1e-6, 1e-6)


"""
class TestLinearWave(unittest.TestCase):


    def test1(self):

        lw = LinearWaves()
        lw.Uc = 7.0
        lw.z_surface = 20.0
        lw.hs = 10.0
        lw.T = 2.0
        lw.z_floor = 0.0
        lw.betaWave = 3.0
        lw.z = np.linspace(0.0, 20.0, 20)

        names, errors = check_gradient(lw)

        tol = 1e-4
        for name, err in zip(names, errors):

            if name in ('d_U[0] / d_z[0]', 'd_U[19] / d_z[19]', 'd_A[0] / d_z[0]', 'd_A[19] / d_z[19]'):
                continue  # the boundaries are not differentiable across bounds. these nodes must not move

            try:
                self.assertLessEqual(err, tol)
            except AssertionError, e:
                print('*** error in:', name)
                raise e


    def test2(self):

        lw = LinearWaves()
        lw.Uc = 5.0
        lw.z_surface = 20.0
        lw.hs = 2.0
        lw.T = 10.0
        lw.z_floor = 0.0
        lw.betaWave = 3.0
        lw.z = np.linspace(-5.0, 50.0, 20)

        names, errors = check_gradient(lw)

        tol = 1e-6
        for name, err in zip(names, errors):

            try:
                self.assertLessEqual(err, tol)
            except AssertionError, e:
                print('*** error in:', name)
                raise e

"""

if __name__ == '__main__':
    unittest.main()



import numpy as np

def fread(fid, nelements, dtype):
     if dtype is np.str:
         dt = np.uint8  # WARNING: assuming 8-bit ASCII for np.str!
     else:
         dt = dtype

     data_array = np.fromfile(fid, dt, nelements)
     data_array.shape = (nelements, 1)

     return data_array

fid = open('./turbsim_default.bts', 'rb');
nffc = 3
fileFmt = 'int16' # kind 2 ?

tmp   = fread( fid, 1, np.int16);        # TurbSim format identifier (should = 7 or 8 if periodic), INT(2)
nz    = fread( fid, 1, np.int32);        # the number of grid points vertically, INT(4)
ny    = fread( fid, 1, np.int32);        # the number of grid points laterally, INT(4)
ntwr  = fread( fid, 1, np.int32);        # the number of tower points, INT(4)
nt    = fread( fid, 1, np.int32);        # the number of time steps, INT(4)

dz    = fread( fid, 1, np.float32);      # grid spacing in vertical direction, REAL(4), in m
dy    = fread( fid, 1, np.float32);      # grid spacing in lateral direction, REAL(4), in m
dt    = fread( fid, 1, np.float32);      # grid spacing in delta time, REAL(4), in m/s
mffws = fread( fid, 1, np.float32);      # the mean wind speed at hub height, REAL(4), in m/s
zHub  = fread( fid, 1, np.float32);      # height of the hub, REAL(4), in m
z1    = fread( fid, 1, np.float32);      # height of the bottom of the grid, REAL(4), in m

Vslope = np.ones(3)
Voffset = np.ones(3)
Vslope[0]  = fread( fid, 1, np.float32); # the U-component slope for scaling, REAL(4)
Voffset[0] = fread( fid, 1, np.float32); # the U-component offset for scaling, REAL(4)
Vslope[1]  = fread( fid, 1, np.float32); # the V-component slope for scaling, REAL(4)
Voffset[1] = fread( fid, 1, np.float32); # the V-component offset for scaling, REAL(4)
Vslope[2]  = fread( fid, 1, np.float32); # the W-component slope for scaling, REAL(4)
Voffset[2] = fread( fid, 1, np.float32); # the W-component offset for scaling, REAL(4)

# Read the description string: "Generated by TurbSim (vx.xx, dd-mmm-yyyy) on dd-mmm-yyyy at hh:mm:ss."

nchar    = fread( fid, 1, np.int32);     # the number of characters in the description string, max 200, INT(4)
asciiINT = fread( fid, int(nchar), np.int8); # the ASCII integer representation of the character string


nPts        = ny*nz;
nv          = nffc*nPts;               # the size of one time step
nvTwr       = nffc*ntwr;
velocity    = np.zeros([int(s) for s in (nt,nffc,ny,nz)])
twrVelocity = np.zeros([int(s) for s in (nt,nffc,ntwr)])

for it in range(nt):
  ip = 0
  v_cnt = fread( fid, int(nv), fileFmt )
  for iz in range(nz):
    for iy in range(ny):
       for k in range(nffc):
          velocity[it,k,iy,iz] = ( v_cnt[ip] - Voffset[k])/Vslope[k] 
          ip = ip + 1
 
print velocity

np.save('velocity', velocity)



from floatingse.instance.floating_instance import FloatingInstance, NPTS, vecOption, Five_strings, Ten_strings
from wisdem.floating.floating_turbine_assembly import FloatingTurbine
from commonse import eps
from commonse.csystem import rotMat_x, rotMat_y, rotMat_z
from rotorse import TURBULENCE_CLASS, TURBINE_CLASS, DRIVETRAIN_TYPE, NREL5MW, DTU10MW
import numpy as np
import offshorebos.wind_obos as wind_obos
import time
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO

NDEL = 0


class FloatingTurbineInstance(FloatingInstance):
    def __init__(self, refStr):
        super(FloatingTurbineInstance, self).__init__()

        if type(refStr) != type(''):
            raise ValueError('Must enter reference turbine name as a string')
        if refStr in Five_strings:
            self.refBlade = NREL5MW()
        elif refStr in Ten_strings:
            self.refBlade = DTU10MW()
        else:
            raise ValueError('Unknown reference turbine name, '+refStr)
            
        # Remove what we don't need from Semi
        self.params.pop('rna_cg', None)
        self.params.pop('rna_mass', None)
        self.params.pop('rna_I', None)
        self.params.pop('rna_moment', None)
        self.params.pop('rna_force', None)
        self.params.pop('sg.Rhub', None)
        self.params.pop('tip_position', None)

        # Environmental parameters
        self.params['air_density'] = self.params['main.windLoads.rho']
        self.params.pop('main.windLoads.rho')
        
        self.params['air_viscosity'] = self.params['main.windLoads.mu']
        self.params.pop('main.windLoads.mu', None)
        
        self.params['water_viscosity'] = self.params['main.waveLoads.mu']
        self.params.pop('main.waveLoads.mu')
        
        self.params['wave_height'] = self.params['Hs']
        self.params.pop('Hs')
        
        self.params['wave_period'] = self.params['T']
        self.params.pop('T', None)
        
        self.params['mean_current_speed'] = self.params['Uc']
        self.params.pop('Uc', None)

        self.params['wind_reference_speed'] = self.params['Uref']
        self.params.pop('Uref', None)
        
        self.params['wind_reference_height'] = self.params['zref']
        self.params.pop('zref')
        
        self.params['shearExp'] = 0.11

        self.params['morison_mass_coefficient'] = self.params['cm']
        self.params.pop('cm', None)
        
        self.params['wind_bottom_height'] = self.params['z0']
        self.params.pop('z0', None)

        self.params['wind_beta'] = self.params['beta']
        self.params.pop('beta', None)
        
        #self.params['wave_beta']                            = 0.0

        self.params['hub_height']                            = 90.0
        
        self.params['gamma_freq']               = 1.1
        self.params['gamma_f']                  = 1.35 # stress
        self.params['gamma_m']               = 1.3 # materials
        self.params['gamma_b']                = 1.1 # buckling
        self.params['gamma_fatigue']                 = 1.35*1.3*1.0
        self.params['gamma_n']             = 1.0 # consequence
      
        self.params['project_lifetime']                      = 20.0
        self.params['number_of_turbines']                    = 20
        self.params['annual_opex']                           = 7e5
        self.params['fixed_charge_rate']                     = 0.12
        self.params['discount_rate']                         = 0.07

        # For RotorSE
        self.params['hubFraction']                           = 0.025
        self.params['bladeLength']                           = 61.5
        self.params['r_max_chord']                           = 0.23577
        self.params['chord_in']                              = np.array([3.2612, 4.5709, 3.3178, 1.4621])
        self.params['theta_in']                              = np.array([13.2783, 7.46036, 2.89317, -0.0878099])
        self.params['precone']                               = 2.5
        self.params['tilt']                                  = 5.0
        self.params['control_Vin']                           = 3.0
        self.params['control_Vout']                          = 25.0
        self.params['machine_rating']                        = 5e6
        self.params['control_minOmega']                      = 0.0
        self.params['control_maxOmega']                      = 12.0
        self.params['control_tsr']                           = 7.55
        self.params['sparT_in']                              = np.array([0.05, 0.047754, 0.045376, 0.031085, 0.0061398])
        self.params['teT_in']                                = np.array([0.1, 0.09569, 0.06569, 0.02569, 0.00569])

        self.params['precurve_in']                           = np.array([0.0, 0.0, 0.0])
        self.params['presweep_in']                           = np.array([0.0, 0.0, 0.0])
        self.params['precurve_tip']                          = 0.0
        self.params['presweep_tip']                          = 0.0
        self.params['yaw']                                   = 0.0
        self.params['nBlades']                               = 3
        self.params['turbine_class']                         = TURBINE_CLASS['I']
        self.params['turbulence_class']                      = TURBULENCE_CLASS['B']
        self.params['drivetrainType']                        = DRIVETRAIN_TYPE['GEARED']
        self.params['gust_stddev']                           = 3
        self.params['control_pitch']                         = 0.0
        self.params['VfactorPC']                             = 0.7
        self.params['pitch_extreme']                         = 0.0
        self.params['azimuth_extreme']                       = 0.0
        self.params['rstar_damage']                          = np.linspace(0.0, 1.0, len(self.refBlade.r)+1)
        self.params['Mxb_damage']                            = eps * np.ones(len(self.refBlade.r)+1)
        self.params['Myb_damage']                            = eps * np.ones(len(self.refBlade.r)+1)
        self.params['strain_ult_spar']                       = 1e-2
        self.params['strain_ult_te']                         = 2*2500*1e-6
        self.params['m_damage']                              = 10.0
        self.params['nSector']                               = 4
        self.params['tiploss']                               = True
        self.params['hubloss']                               = True
        self.params['wakerotation']                          = True 
        self.params['usecd']                                 = True
        self.params['AEP_loss_factor']                       = 1.0
        self.params['dynamic_amplication_tip_deflection']    = 1.35
        self.params['shape_parameter']                       = 0.0

        # For RNA
        self.params['hub_mass']    = 56.780e3
        self.params['nac_mass']    = 240e3
        self.params['hub_cm']      = np.array([-5.01910, 0.0, 1.96256])
        self.params['nac_cm']      = np.array([1.9, 0.0, 1.75])
        self.params['hub_I']       = self.params['hub_mass']*1.75**2. * np.r_[(2./3.), (5./12.), (5./12.), np.zeros(3)]
        self.params['nac_I']       = np.array([7.77616624894e7, 8.34033992e+05, 8.34033992e+05, 0.0, 2.05892434e+05, 0.0])
        self.params['downwind']    = False
        self.params['rna_weightM'] = True

        # For turbine costs
        self.params['blade_mass_cost_coeff']                 = 13.08
        self.params['hub_mass_cost_coeff']                   = 3.8
        self.params['pitch_system_mass_cost_coeff']          = 22.91
        self.params['spinner_mass_cost_coeff']               = 23.0
        self.params['lss_mass_cost_coeff']                   = 12.6
        self.params['bearings_mass_cost_coeff']              = 6.35
        self.params['gearbox_mass_cost_coeff']               = 17.4
        self.params['hss_mass_cost_coeff']       = 8.25
        self.params['generator_mass_cost_coeff']             = 17.43
        self.params['bedplate_mass_cost_coeff']              = 4.5
        self.params['yaw_mass_cost_coeff']            = 11.01
        self.params['vs_electronics_mass_cost_coeff']   = 26.5
        self.params['hvac_mass_cost_coeff']     = 163.95
        self.params['cover_mass_cost_coeff']         = 7.61
        self.params['elec_connec_machine_rating_cost_coeff'] = 40.0
        self.params['platforms_mass_cost_coeff']     = 8.7
        self.params['base_hardware_cost_coeff']              = 0.7
        self.params['transformer_mass_cost_coeff']           = 26.5
        self.params['tower_mass_cost_coeff']                 = 3.20
        self.params['hub_assemblyCostMultiplier']            = 0.0
        self.params['hub_overheadCostMultiplier']            = 0.0
        self.params['nacelle_assemblyCostMultiplier']        = 0.0
        self.params['nacelle_overheadCostMultiplier']        = 0.0
        self.params['tower_assemblyCostMultiplier']          = 0.0
        self.params['tower_overheadCostMultiplier']          = 0.0
        self.params['turbine_assemblyCostMultiplier']        = 0.0
        self.params['turbine_overheadCostMultiplier']        = 0.0
        self.params['hub_profitMultiplier']                  = 0.0
        self.params['nacelle_profitMultiplier']              = 0.0
        self.params['tower_profitMultiplier']                = 0.0
        self.params['turbine_profitMultiplier']              = 0.0
        self.params['hub_transportMultiplier']               = 0.0
        self.params['nacelle_transportMultiplier']           = 0.0
        self.params['tower_transportMultiplier']             = 0.0
        self.params['turbine_transportMultiplier']           = 0.0
        self.params['offshore']                              = True
        self.params['crane']                                 = False
        self.params['bearing_number']       = 2
        self.params['bedplate_mass']        = 93090.6
        self.params['controls_cost_base']   = np.array([35000.0,55900.0])
        self.params['controls_esc']         = 1.5
        self.params['crane_cost']           = 0.0
        self.params['elec_connec_cost_esc'] = 1.5
        self.params['gearbox_mass']         = 30237.60
        self.params['generator_mass']       = 16699.85
        self.params['hss_mass']             = 1492.45
        self.params['hvac_mass']            = 1e3
        self.params['lss_mass']             = 31257.3
        self.params['main_bearing_mass']    = 9731.41 / 2
        self.params['cover_mass']           = 1e3
        self.params['platforms_mass']       = 1e3
        self.params['pitch_system_mass']    = 17004.0
        self.params['spinner_mass']         = 1810.5
        self.params['transformer_mass']     = 1e3
        self.params['vs_electronics_mass']  = 1e3
        self.params['yaw_mass']             = 11878.24
        
        # Offshore BOS
        # Turbine / Plant parameters
        self.params['nacelleL']                              = -np.inf
        self.params['nacelleW']                              = -np.inf
        self.params['distShore']                             = 90.0
        self.params['distPort']                              = 90.0
        self.params['distPtoA']                              = 90.0
        self.params['distAtoS']                              = 90.0
        self.params['substructure']                          = wind_obos.Substructure.SEMISUBMERSIBLE
        self.params['anchor']                                = wind_obos.Anchor.DRAGEMBEDMENT
        self.params['turbInstallMethod']                     = wind_obos.TurbineInstall.INDIVIDUAL
        self.params['towerInstallMethod']                    = wind_obos.TowerInstall.ONEPIECE
        self.params['installStrategy']                       = wind_obos.InstallStrategy.PRIMARYVESSEL
        self.params['cableOptimizer']                        = False
        self.params['buryDepth']                             = 2.0
        self.params['arrayY']                                = 9.0
        self.params['arrayX']                                = 9.0
        self.params['substructCont']                         = 0.30
        self.params['turbCont']                              = 0.30
        self.params['elecCont']                              = 0.30
        self.params['interConVolt']                          = 345.0
        self.params['distInterCon']                          = 3.0
        self.params['scrapVal']                              = 0.0
        #General']                                           = , 
        self.params['inspectClear']                          = 2.0
        self.params['plantComm']                             = 0.01
        self.params['procurement_contingency']               = 0.05
        self.params['install_contingency']                   = 0.30
        self.params['construction_insurance']                = 0.01
        self.params['capital_cost_year_0']                   = 0.20
        self.params['capital_cost_year_1']                   = 0.60
        self.params['capital_cost_year_2']                   = 0.10
        self.params['capital_cost_year_3']                   = 0.10
        self.params['capital_cost_year_4']                   = 0.0
        self.params['capital_cost_year_5']                   = 0.0
        self.params['tax_rate']                              = 0.40
        self.params['interest_during_construction']          = 0.08
        #Substructure & Foundation']                         = , 
        self.params['mpileCR']                               = 2250.0
        self.params['mtransCR']                              = 3230.0
        self.params['jlatticeCR']                            = 4680.0
        self.params['jtransCR']                              = 4500.0
        self.params['jpileCR']                               = 2250.0
        self.params['jlatticeA']                             = 26.0
        self.params['jpileL']                                = 47.50
        self.params['jpileD']                                = 1.60
        self.params['ssHeaveCR']                             = 6250.0
        self.params['scourMat']                              = 250000.0
        self.params['number_install_seasons']                = 1.0
        self.params['deaFixLeng']                = 0.0
        #Electrical Infrastructure']                         = , 
        self.params['pwrFac']                                = 0.95
        self.params['buryFac']                               = 0.10
        self.params['catLengFac']                            = 0.04
        self.params['exCabFac']                              = 0.10
        self.params['subsTopFab']                            = 14500.0
        self.params['subsTopDes']                            = 4500000.0
        self.params['topAssemblyFac']                        = 0.075
        self.params['subsJackCR']                            = 6250.0
        self.params['subsPileCR']                            = 2250.0
        self.params['dynCabFac']                             = 2.0
        self.params['shuntCR']                               = 35000.0
        self.params['highVoltSG']                            = 950000.0
        self.params['medVoltSG']                             = 500000.0
        self.params['backUpGen']                             = 1000000.0
        self.params['workSpace']                             = 2000000.0
        self.params['otherAncillary']                        = 3000000.0
        self.params['mptCR']                                 = 12500.0
        self.params['arrVoltage']                            = 33.0
        self.params['cab1CR']                                = 185.889
        self.params['cab2CR']                                = 202.788
        self.params['cab1CurrRating']                        = 300.0
        self.params['cab2CurrRating']                        = 340.0
        self.params['arrCab1Mass']                           = 20.384
        self.params['arrCab2Mass']                           = 21.854
        self.params['cab1TurbInterCR']                       = 8410.0
        self.params['cab2TurbInterCR']                       = 8615.0
        self.params['cab2SubsInterCR']                       = 19815.0
        self.params['expVoltage']                            = 220.0
        self.params['expCurrRating']                         = 530.0
        self.params['expCabMass']                            = 71.90
        self.params['expCabCR']                              = 495.411
        self.params['expSubsInterCR']                        = 57500.0
        # Vector inputs
        #self.params['arrayCables']                          = [33, 66]
        #self.params['exportCables']                         = [132, 220]
        #Assembly & Installation',
        self.params['moorTimeFac']                           = 0.005
        self.params['moorLoadout']                           = 5.0
        self.params['moorSurvey']                            = 4.0
        self.params['prepAA']                                = 168.0
        self.params['prepSpar']                              = 18.0
        self.params['upendSpar']                             = 36.0
        self.params['prepSemi']                              = 12.0
        self.params['turbFasten']                            = 8.0
        self.params['boltTower']                             = 7.0
        self.params['boltNacelle1']                          = 7.0
        self.params['boltNacelle2']                          = 7.0
        self.params['boltNacelle3']                          = 7.0
        self.params['boltBlade1']                            = 3.50
        self.params['boltBlade2']                            = 3.50
        self.params['boltRotor']                             = 7.0
        self.params['vesselPosTurb']                         = 2.0
        self.params['vesselPosJack']                         = 8.0
        self.params['vesselPosMono']                         = 3.0
        self.params['subsVessPos']                           = 6.0
        self.params['monoFasten']                            = 12.0
        self.params['jackFasten']                            = 20.0
        self.params['prepGripperMono']                       = 1.50
        self.params['prepGripperJack']                       = 8.0
        self.params['placePiles']                            = 12.0
        self.params['prepHamMono']                           = 2.0
        self.params['prepHamJack']                           = 2.0
        self.params['removeHamMono']                         = 2.0
        self.params['removeHamJack']                         = 4.0
        self.params['placeTemplate']                         = 4.0
        self.params['placeJack']                             = 12.0
        self.params['levJack']                               = 24.0
        self.params['hamRate']                               = 20.0
        self.params['placeMP']                               = 3.0
        self.params['instScour']                             = 6.0
        self.params['placeTP']                               = 3.0
        self.params['groutTP']                               = 8.0
        self.params['tpCover']                               = 1.50
        self.params['prepTow']                               = 12.0
        self.params['spMoorCon']                             = 20.0
        self.params['ssMoorCon']                             = 22.0
        self.params['spMoorCheck']                           = 16.0
        self.params['ssMoorCheck']                           = 12.0
        self.params['ssBall']                                = 6.0
        self.params['surfLayRate']                           = 375.0
        self.params['cabPullIn']                             = 5.50
        self.params['cabTerm']                               = 5.50
        self.params['cabLoadout']                            = 14.0
        self.params['buryRate']                              = 125.0
        self.params['subsPullIn']                            = 48.0
        self.params['shorePullIn']                           = 96.0
        self.params['landConstruct']                         = 7.0
        self.params['expCabLoad']                            = 24.0
        self.params['subsLoad']                              = 60.0
        self.params['placeTop']                              = 24.0
        self.params['pileSpreadDR']                          = 2500.0
        self.params['pileSpreadMob']                         = 750000.0
        self.params['groutSpreadDR']                         = 3000.0
        self.params['groutSpreadMob']                        = 1000000.0
        self.params['seaSpreadDR']                           = 165000.0
        self.params['seaSpreadMob']                          = 4500000.0
        self.params['compRacks']                             = 1000000.0
        self.params['cabSurveyCR']                           = 240.0
        self.params['cabDrillDist']                          = 500.0
        self.params['cabDrillCR']                            = 3200.0
        self.params['mpvRentalDR']                           = 72000.0
        self.params['diveTeamDR']                            = 3200.0
        self.params['winchDR']                               = 1000.0
        self.params['civilWork']                             = 40000.0
        self.params['elecWork']                              = 25000.0
        #Port & Staging']                                    = , 
        self.params['nCrane600']                             = 0.0
        self.params['nCrane1000']                            = 0.0
        self.params['crane600DR']                            = 5000.0
        self.params['crane1000DR']                           = 8000.0
        self.params['craneMobDemob']                         = 150000.0
        self.params['entranceExitRate']                      = 0.525
        self.params['dockRate']                              = 3000.0
        self.params['wharfRate']                             = 2.75
        self.params['laydownCR']                             = 0.25
        #Engineering & Management']                          = , 
        self.params['estEnMFac']                             = 0.04
        #Development']                                       = , 
        self.params['preFEEDStudy']                          = 5000000.0
        self.params['feedStudy']                             = 10000000.0
        self.params['stateLease']                            = 250000.0
        self.params['outConShelfLease']                      = 1000000.0
        self.params['saPlan']                                = 500000.0
        self.params['conOpPlan']                             = 1000000.0
        self.params['nepaEisMet']                            = 2000000.0
        self.params['physResStudyMet']                       = 1500000.0
        self.params['bioResStudyMet']                        = 1500000.0
        self.params['socEconStudyMet']                       = 500000.0
        self.params['navStudyMet']                           = 500000.0
        self.params['nepaEisProj']                           = 5000000.0
        self.params['physResStudyProj']                      = 500000.0
        self.params['bioResStudyProj']                       = 500000.0
        self.params['socEconStudyProj']                      = 200000.0
        self.params['navStudyProj']                          = 250000.0
        self.params['coastZoneManAct']                       = 100000.0
        self.params['rivsnHarbsAct']                         = 100000.0
        self.params['cleanWatAct402']                        = 100000.0
        self.params['cleanWatAct404']                        = 100000.0
        self.params['faaPlan']                               = 10000.0
        self.params['endSpecAct']                            = 500000.0
        self.params['marMamProtAct']                         = 500000.0
        self.params['migBirdAct']                            = 500000.0
        self.params['natHisPresAct']                         = 250000.0
        self.params['addLocPerm']                            = 200000.0
        self.params['metTowCR']                              = 11518.0
        self.params['decomDiscRate']                         = 0.03

        self.params['dummy_mass']                            = eps


    def set_reference(self, instr):
        if instr.upper() in Five_strings:
            myref = NREL5MW()

            self.params['hub_mass'] = 56.780e3
            self.params['nac_mass'] = 240e3
            self.params['hub_cm']   = np.array([-5.01910, 0.0, 1.96256])
            self.params['nac_cm']   = np.array([1.9, 0.0, 1.75])
            self.params['hub_I']    = self.params['hub_mass']*1.75**2. * np.r_[(2./3.), (5./12.), (5./12.), np.zeros(3)]
            self.params['nac_I']    = np.array([7.77616624894e7, 8.34033992e+05, 8.34033992e+05, 0.0, 2.05892434e+05, 0.0])
            self.params['rna_weightM'] = True
            
        elif instr.upper() in Ten_strings:
            myref = DTU10MW()

            self.params['hub_mass'] = 105520.0
            self.params['nac_mass'] = 446036.25
            self.params['hub_cm']   = np.array([-7.1, 0.0, 2.75])
            self.params['nac_cm']   = np.array([2.69, 0.0, 2.40])
            self.params['hub_I']    = self.params['hub_mass']*2.152**2. * np.r_[(2./3.), (5./12.), (5./12.), np.zeros(3)]
            self.params['nac_I']    = self.params['nac_mass']*(1./12.) * np.r_[(10**2+10**2), (10**2+15**2), (15**2+10**2), np.zeros(3)]
            self.params['rna_weightM'] = True

        # Set blade/rotor values from reference definition in RotorSE
        self.params['hubFraction']      = myref.hubFraction
        self.params['bladeLength']      = myref.bladeLength
        self.params['precone']          = myref.precone
        self.params['tilt']             = myref.tilt
        self.params['nBlades']          = myref.nBlades
        self.params['downwind']         = myref.downwind
        self.params['r_max_chord']      = myref.r_max_chord
        self.params['chord_in']         = myref.chord
        self.params['theta_in']         = myref.theta
        self.params['precurve_in']      = myref.precurve
        self.params['presweep_in']      = myref.presweep
        self.params['sparT_in']         = myref.spar_thickness
        self.params['teT_in']           = myref.te_thickness
        self.params['hub_height']       = myref.hub_height
        self.params['turbine_class']    = myref.turbine_class
        self.params['wind_reference_height'] = myref.hub_height
        self.params['control_Vin']      = myref.control_Vin
        self.params['control_Vout']     = myref.control_Vout
        self.params['control_minOmega'] = myref.control_minOmega
        self.params['control_maxOmega'] = myref.control_maxOmega
        self.params['control_tsr']      = myref.control_tsr
        self.params['control_pitch']    = myref.control_pitch
        self.params['machine_rating']   = myref.rating
        self.params['drivetrainType']   = myref.drivetrain

        super(FloatingTurbineInstance, self).set_reference(instr)
        
        
    def get_assembly(self): return FloatingTurbine(self.refBlade)

    def add_objective(self, varname='lcoe', scale=1.0):
        super(FloatingTurbineInstance, self).add_objective(varname, scale)
            
        
    def get_constraints(self):
        conList = super(FloatingTurbineInstance, self).get_constraints()

        conList.extend( [['Pn_margin', None, 1.0, None],
                         ['P1_margin', None, 1.0, None],
                         ['Pn_margin_cfem', None, 1.0, None],
                         ['P1_margin_cfem', None, 1.0, None],
                         ['rotor_strain_sparU', -1.0, None, None],
                         ['rotor_strain_sparL', None, 1.0, None],
                         ['rotor_strain_teU', -1.0, None, None],
                         ['rotor_strain_teL', None, 1.0, None],
                         ['rotor_buckling_sparU', None, 1.0, None],
                         ['rotor_buckling_sparL', None, 1.0, None],
                         ['rotor_buckling_teU', None, 1.0, None],
                         ['rotor_buckling_teL', None, 1.0, None],
                         ['rotor_damage_sparU', None, 0.0, None],
                         ['rotor_damage_sparL', None, 0.0, None],
                         ['rotor_damage_teU', None, 0.0, None],
                         ['rotor_damage_teL', None, 0.0, None],
                         #['frequency1P_margin_low', None, 1.0, None],
                         #['frequency1P_margin_high', 1.0, None, None],
                         #['frequency3P_margin_low', None, 1.0, None],
                         #['frequency3P_margin_high', 1.0, None, None],
                         #['tip_deflection_ratio', None, 1.0, None],
                         ['ground_clearance', 20.0, None, None],
        ])
        return conList

    def draw_rna(self, fig):
        from mayavi import mlab
        if fig is None: fig=self.init_figure()

        # Quantities from input and output simulatioin parameters
        r_cylinder   = self.refBlade.r_cylinder
        bladeLength  = self.params['bladeLength']
        hubD         = 2*self.prob['Rhub']
        nblade       = self.params['nBlades']
        pitch        = 0.0
        precone      = self.params['precone']
        tilt         = self.params['tilt']
        hubH         = self.params['hub_height']
        cm_hub       = self.prob['hub_cm']
        cm_hub[-1]  += hubH
        chord        = self.prob['chord']
        thick        = self.prob['chord'] / self.refBlade.chord_ref
        twist        = self.prob['theta']
        precurve     = self.prob['precurve']
        presweep     = self.prob['presweep']
        le_loc       = self.refBlade.le_location

        # Rotation matrices
        T_tilt    = rotMat_y(np.deg2rad(tilt))
        T_precone = rotMat_y(np.deg2rad(-precone))
        T_pitch   = rotMat_z(np.deg2rad(-pitch))

        # Spanwise coordinates
        r_blade = self.prob['r_pts']

        # Airfoil coordinates
        afcoord = self.refBlade.getAirfoilCoordinates()

        # Assemble airfoil coordinates along the span
        # Not flip x and y and flip sign to be consistent with global coordinate system at pitch=0:
        # +x downstream
        # +y towards TE
        # +z towards sky
        X = []
        Y = []
        for k in range(len(r_blade)):
            thisy = afcoord[k][:,0].copy()
            thisx = afcoord[k][:,1].copy()

            # Pre-twist modifications
            thisy -= le_loc[k]
            thisx *= chord[k] * thick[k]
            thisy *= chord[k]

            # Chord scaling and twist rotation
            thismat = np.asmatrix( np.c_[thisx, thisy] ).T
            T_th    = rotMat_z( np.deg2rad(-twist[k]) )[:2,:2]
            thismat = np.asarray( (T_th * thismat).T )

            if k==0:
                X = thismat[:,0]
                Y = thismat[:,1]
            else:
                X = np.c_[X, thismat[:,0] + precurve[k]]
                Y = np.c_[Y, thismat[:,1] + presweep[k]]

        # Set Z-positions
        Z = r_blade[np.newaxis,:] * np.ones(X.shape)

        # Create plot of blades
        bladeAng = np.linspace(0, 2*np.pi, nblade+1)[:nblade]
        orig = X.shape
        for a in bladeAng:
            T_tot   = T_tilt * rotMat_x(a) * T_precone * T_pitch
            thismat = np.asmatrix( np.c_[X.flatten(), Y.flatten(), Z.flatten()] ).T
            thismat = np.asarray( (T_tot * thismat).T ) + cm_hub[np.newaxis,:]
            Xplot, Yplot, Zplot = thismat[:,0], thismat[:,1], thismat[:,2]
            mlab.mesh(Xplot.reshape(orig), Yplot.reshape(orig), Zplot.reshape(orig), color=(1,1,1), figure=fig)

        # Now do hub
        npts    = 30
        rk      = 0.5*hubD*np.ones(npts)
        th      = np.linspace(0,2*np.pi,npts)
        x       = np.linspace(-0.5, 0.5, npts) * 1.5*chord[0]
        R, TH   = np.meshgrid(rk, th)
        X, _    = np.meshgrid(x, th)
        Y       = R*np.cos(TH)
        Z       = R*np.sin(TH)
        orig    = X.shape
        thismat = np.asmatrix( np.c_[X.flatten(), Y.flatten(), Z.flatten()] ).T
        thismat = np.asarray( (T_tilt * thismat).T ) + cm_hub[np.newaxis,:]
        Xplot, Yplot, Zplot = thismat[:,0], thismat[:,1], thismat[:,2]
        mlab.mesh(Xplot.reshape(orig), Yplot.reshape(orig), Zplot.reshape(orig), color=(0.9,)*3, figure=fig)

        ph      = np.linspace(0, 0.5*np.pi, npts) + np.pi
        PH,_    = np.meshgrid(ph,th)
        Y       = R*np.cos(TH)*np.sin(PH)
        Z       = R*np.sin(TH)*np.sin(PH)
        X       = R*np.cos(PH) - 0.75*chord[0]
        orig    = X.shape
        thismat = np.asmatrix( np.c_[X.flatten(), Y.flatten(), Z.flatten()] ).T
        thismat = np.asarray( (T_tilt * thismat).T ) + cm_hub[np.newaxis,:]
        Xplot, Yplot, Zplot = thismat[:,0], thismat[:,1], thismat[:,2]
        mlab.mesh(Xplot.reshape(orig), Yplot.reshape(orig), Zplot.reshape(orig), color=(0.9,)*3, figure=fig)

        # Now do nacelle
        nacW   = nacH = hubD + 2.0
        nacL   = nacW + 5.0
        cm_nac = cm_hub + 0.5*np.array([nacL, 0.0, 0.0])
        cm_nac[0] += 0.75*chord[0]
        xx     = np.array([-0.5, 0.0, 0.5])
        PX,PY  = np.meshgrid(xx, xx)
        PZ     = np.ones(PX.shape)
        # Top and bottom
        mlab.mesh(PX*nacL + cm_nac[0], PY*nacW + cm_nac[1], -PZ*0.5*nacH + cm_nac[2], color=(0.9,)*3, figure=fig)
        mlab.mesh(PX*nacL + cm_nac[0], PY*nacW + cm_nac[1],  PZ*0.5*nacH + cm_nac[2], color=(0.9,)*3, figure=fig)
        # Sides
        mlab.mesh(PX*nacL + cm_nac[0], -PZ*nacW*0.5 + cm_nac[1], PY*nacH + cm_nac[2], color=(0.9,)*3, figure=fig)
        mlab.mesh(PX*nacL + cm_nac[0],  PZ*nacW*0.5 + cm_nac[1], PY*nacH + cm_nac[2], color=(0.9,)*3, figure=fig)
        # Front and Back
        mlab.mesh(-PZ*nacL*0.5 + cm_nac[0], PX*nacW + cm_nac[1], PY*nacH + cm_nac[2], color=(0.9,)*3, figure=fig)
        mlab.mesh( PZ*nacL*0.5 + cm_nac[0], PX*nacW + cm_nac[1], PY*nacH + cm_nac[2], color=(0.9,)*3, figure=fig)



        
